import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Queue;
/**
 * Searching the graph, started from a node.
 *	Normal breath-first-search
 *
 */
public class BreadthFirstSearch {

   private HashMap<GraphNode, Integer> visitedNodes = null;
   private Queue<GraphNode> queue = new LinkedList<GraphNode>();
   
   public BreadthFirstSearch(GraphNode node){
      this();
      startSearch(node);
   }
   public BreadthFirstSearch(){
      visitedNodes = new HashMap<GraphNode, Integer>();
   }
   public Queue<GraphNode> getWorkList(){
      return queue;
   }
   public HashMap<GraphNode, Integer> getSearchResult(){
    return visitedNodes;  
   }
   /**
    * Starts the Search
    * @param node starting node for our search
    * @return Hashmap with the depth for each node
    */
   public HashMap<GraphNode, Integer> startSearch(GraphNode node){
      visitedNodes = new HashMap<GraphNode, Integer>();
      queue = new LinkedList<GraphNode>();
      mark(node, 0);
      search(node, 0);
      return visitedNodes;
   }
   /**
    * recursive search worker - method
    * @param node beginning search on this node
    * @param depth depth of this node
    */
   private void search(GraphNode node, int depth){
      if(visitedNodes.get(node) < depth )
         return;
            
      //Our next used depth
      int nextDepth = depth + 1;
      //from now on we go a level deeper!
      //Imortant: mark all nodes first!
      for(Iterator child = node.getNeighbours(); child.hasNext();){
         GraphNode aNode = (GraphNode)child.next();
         mark(aNode, nextDepth);
         //queue.offer(aNode);
      }
      //In the second iteration we check all the subnodes
      for(Iterator child = node.getNeighbours(); child.hasNext();)
         search((GraphNode)child.next(), nextDepth);
   }
   
   //Marks the node, if it has no mark until now
   private void mark(GraphNode node, int depth) {
      if(visitedNodes.get(node) != null )
         return;
      //Add our nodes in the order they are marked
      queue.offer(node);
      visitedNodes.put(node, depth);
   }
}
