import java.awt.GridLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * our control panel for different settings. *
 */
public class ControlPanel extends JPanel {

	private RadialGraphBuilder builder;

	private JSlider jSliderFisheye;

	private JSlider jSliderRadius;

	private JPanel jPanelRadioButtons;
	
	private JCheckBox jCheckBoxChroma;

	public ControlPanel(RadialGraphBuilder builder) {
		this.builder = builder;

		initialize();

	}

	private void initialize() {
		this.setLayout(new BoxLayout(this, BoxLayout.PAGE_AXIS));
		JPanel panel = new JPanel();
		panel.add(new JLabel("Fisheye"));
		panel.add(getFisheyeSlider());
		this.add(panel);

		panel = new JPanel();
		panel.add(new JLabel("Radius"));
		panel.add(getRadiusSlider());
		this.add(panel);

		panel = new JPanel();
		panel.add(new JLabel("Node Size"));
		panel.add(getRadioButtons());
		this.add(panel);
		
		panel = new JPanel();
		panel.add(new JLabel("Chroma Depth"));
		panel.add(getChromaCheckbox());
		this.add(panel);
	}

	private JSlider getFisheyeSlider() {
		if (jSliderFisheye == null) {
			jSliderFisheye = new JSlider(0, 100, 100);
			jSliderFisheye.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					builder
							.createCircles(((float) jSliderFisheye.getValue() / 100));
				}
			});
			jSliderFisheye.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseReleased(MouseEvent e) {
					super.mouseReleased(e);

					builder.rebuildGraph();
				}
			});
		}
		return jSliderFisheye;
	}

	private JSlider getRadiusSlider() {
		if (jSliderRadius == null) {
			jSliderRadius = new JSlider(50, 150, 100);
			jSliderRadius.addChangeListener(new ChangeListener() {
				public void stateChanged(ChangeEvent e) {
					builder.createCircles(jSliderRadius.getValue());
				}
			});
			jSliderRadius.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseReleased(MouseEvent e) {
					super.mouseReleased(e);

					builder.rebuildGraph();
				}
			});
		}
		return jSliderRadius;
	}

	private JPanel getRadioButtons() {
		if (jPanelRadioButtons == null) {
			// Create the radio buttons.
			JRadioButton buttonPhotosets = new JRadioButton("Photosets");
			buttonPhotosets.setSelected(true);

			JRadioButton buttonTags = new JRadioButton("Tags");
			JRadioButton buttonContacts = new JRadioButton("Contacts");
			JRadioButton buttonConstant = new JRadioButton("Constant");

			// Group the radio buttons.
			ButtonGroup group = new ButtonGroup();
			group.add(buttonPhotosets);
			group.add(buttonTags);
			group.add(buttonContacts);
			group.add(buttonConstant);

			// Item listener
			ItemListener buttonListener = new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					JRadioButton button = (JRadioButton) e.getItem();
					if (button.isSelected()) {
						if(button.getText() == "Photosets") {
							builder.setNodeSizeMethod(FlickrUser.PHOTOSETS);
						}
						else if(button.getText() == "Tags") {
							builder.setNodeSizeMethod(FlickrUser.TAGS);
						}
						else if(button.getText() == "Contacts") {
							builder.setNodeSizeMethod(FlickrUser.CONTACTS);
						}
						else if(button.getText() == "Constant") {
							builder.setNodeSizeMethod(FlickrUser.CONSTANT);
						}
					}
				}
			};

			buttonPhotosets.addItemListener(buttonListener);
			buttonTags.addItemListener(buttonListener);
			buttonContacts.addItemListener(buttonListener);
			buttonConstant.addItemListener(buttonListener);

			jPanelRadioButtons = new JPanel(new GridLayout(0, 1));
			jPanelRadioButtons.add(buttonPhotosets);
			jPanelRadioButtons.add(buttonTags);
			jPanelRadioButtons.add(buttonContacts);
			jPanelRadioButtons.add(buttonConstant);

		}
		return jPanelRadioButtons;
	}
	
	private JCheckBox getChromaCheckbox() {
		if(jCheckBoxChroma == null) {
			jCheckBoxChroma = new JCheckBox();
			
			ItemListener buttonListener = new ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					JCheckBox button = (JCheckBox) e.getItem();
					if (button.isSelected()) {
						builder.setChromaDepth(true);
					} else {
						builder.setChromaDepth(false);
					}
				}
			};
			
			jCheckBoxChroma.addItemListener(buttonListener);
		}
		return jCheckBoxChroma;
	}
}
