import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.net.URL;
import java.util.*;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import com.aetrion.flickr.*;
import com.aetrion.flickr.auth.*;
import com.aetrion.flickr.contacts.Contact;
import com.aetrion.flickr.contacts.ContactsInterface;
import com.aetrion.flickr.people.PeopleInterface;
import com.aetrion.flickr.people.User;
import com.aetrion.flickr.photos.Photo;
import com.aetrion.flickr.photosets.Photoset;
import com.aetrion.flickr.tags.Tag;

import edu.stanford.ejalbert.BrowserLauncher;
import edu.stanford.ejalbert.BrowserLauncherRunner;

/**
 * Is collecting Flickr-Data form the web-api
 */
public class FlickrTest {
	static String restHost = "www.flickr.com";

	Flickr flickr;

	REST rest;

	RequestContext requestContext;

	AuthInterface authInterface;

	String frob = "";

	String filename = "flickr.properties";

	String apiKey;

	String sharedSecret;

	String authToken;

	Properties properties = null;

	int maxContacts = 4;

	int maxDepth = 5;

	FlickrUserData userData = null;

	public FlickrTest() throws ParserConfigurationException, IOException,
			SAXException, FlickrException {
		try {
			properties = new Properties(System.getProperties());
			loadProperties();
		} catch (IOException e) {
			System.err.println("Couldn't load properties file");
			e.printStackTrace();
		}

		// authenticate flickr
		authFlickr();

		userData = new FlickrUserData();

		// load all the data from XML
		//userData.load();

		// load the data from Flickr and save it to XML
		 loadUserData("mbuchetics", 0);
		 userData.save();
       
       userData = new FlickrUserData();
       //to make sure everything is saved correctly
       userData.load();
       userData.save();

		// debug output
		userData.printDebug();

	}

	/**
	 * loads the data from given user.
	 * Recursivly loads all his friends until given depth is reached
	 * @param userName current user
	 * @param depth depth of user
	 * @return valid user?
	*/
	@SuppressWarnings( { "unchecked" })
	public boolean loadUserData(String userName, int depth) throws IOException,
			SAXException, FlickrException {
		// find user
		User user = null;
		try {
			user = flickr.getPeopleInterface().findByUsername(userName);
		} catch (FlickrException e) {
			System.err.println("Username not found");
			// e.printStackTrace();
			return false;
		}

		// get contacts list
		ArrayList arrayList = new ArrayList();
		ArrayList<Contact> contacts = arrayList;
		contacts = (ArrayList) flickr.getContactsInterface().getPublicList(
				user.getId());

		// get tags list
		ArrayList<Tag> tags = arrayList;
		tags = (ArrayList) flickr.getTagsInterface().getListUser(user.getId());

		// get photosets list
		ArrayList<Photoset> photosets = arrayList;
		photosets = (ArrayList) flickr.getPhotosetsInterface().getList(
				user.getId()).getPhotosets();

		// download and save user icon
		String iconURL = "http://flickr.com/buddyicons/" + user.getId()
				+ ".jpg";
		System.out.println(iconURL);

		Toolkit toolkit = Toolkit.getDefaultToolkit();
		BufferedImage image = toBufferedImage(toolkit
				.getImage(new URL(iconURL)));

		File td = new File("data/" + userName);
		td.mkdir();
		File outfile = new File("data/" + userName + "/icon.jpg");
		ImageIO.write(image, "jpg", outfile);

		System.out.println(depth + ": " + userName + "   tags: " + tags.size()
				+ " sets: " + photosets.size() + " contacts: "
				+ contacts.size());

		FlickrUser myFlickrUser = new FlickrUser(userName);
		myFlickrUser.nrPhotoSets = photosets.size();
		myFlickrUser.nrTags = tags.size();

		userData.addUser(userName, myFlickrUser);
		
		boolean flag;

		// visit all the contacts, funky recursion
		for (int i = 0; i < maxContacts && i < contacts.size(); i++) {
			Contact c = contacts.get(i);

			if (depth < maxDepth) {
				// Username not stored yet, load it
				if (userData.containsUser(c.getUsername()) == false) {
					if (loadUserData(c.getUsername(), depth + 1) == true) {
						myFlickrUser.addContact(userData.getUser(c.getUsername()));
						System.out.println("* " + c.getUsername()
								+ " added to " + userName + "'s contact list");
					} else {
						// Username couldn't be found, shouldn't happen usually
					}
				}
				// Username already stored, just point to it				
				else {					
					// not really right, but it gets rid of a lot of interconnections
					flag =false;
					for(FlickrUser contactUser : myFlickrUser.getContacts()) {
						for(FlickrUser contactContactUser : contactUser.getContacts()) {
							if(contactContactUser.getUserName() == userName)
							{
								myFlickrUser.getContacts().add(userData.getUser(c.getUsername()));
								System.out.println("* " + c.getUsername() + " added to "
									+ userName + "'s contact list");
								flag = true;
								break;
							}
						}
						if(flag == true)
							break;
					}
					// this would be the correct code:
//					myFlickrUser.getContacts().add(userData.getUser(c.getUsername()));
//					System.out.println("* " + c.getUsername() + " added to "
//							+ userName + "'s contact list");
				}
			}

			else if (depth == maxDepth
					&& userData.containsUser(c.getUsername()) == true) {
				myFlickrUser.getContacts().add(userData.getUser(c.getUsername()));
				System.out.println("* " + (depth + 1) + " " + c.getUsername());
			}
		}

		return true;
	}

	// Flickr Authentication
	// since we only read information that is available to the public we don't
	// need special user authentication (where the user would have to give us
	// permission
	// to read/write it's data). Therefore we only need to set our API key and
	// the
	// shared secret.
	private void authFlickr() throws ParserConfigurationException, IOException,
			SAXException, FlickrException {
		rest = new REST();
		rest.setHost(restHost);
		flickr = new Flickr(apiKey, rest);
		Flickr.debugStream = false;

		requestContext = RequestContext.getRequestContext();
		requestContext.setSharedSecret(sharedSecret);

		// following code is heavy authentication, for private read/write
		// permission
		// we don't need this

		// authInterface = flickr.getAuthInterface();
		// // Try and see if the user already authenticated the API use
		// if (checkAuth() == true) {
		// System.out.println("Cached auth worked");
		// } else {
		// System.out.println("Cached auth failed");
		//
		// if (doAuth() == false)
		// System.exit(0);
		//
		// saveProperties();
		// }
		//
		// // Authentication worked, get some user data
		// Auth auth = RequestContext.getRequestContext().getAuth();
		//
		// System.out.println("Authentication success");
		// System.out.println("Token: " + auth.getToken());
		// System.out.println("nsid: " + auth.getUser().getId());
		// System.out.println("Realname: " + auth.getUser().getRealName());
		// System.out.println("Username: " + auth.getUser().getUsername());
		// System.out.println("Count: " + auth.getUser().getLocation());
		// System.out.println("Permission: " + auth.getPermission().getType());
	}

	// If we have a authentication token stored we check if it works
	// so the user doesn't has to give his permission on every application
	// launch
	// we don't need this here, we only use public stuff
	private boolean checkAuth() {
		if (properties.getProperty("token") == null) {
			return false;
		} else {
			Auth auth;
			try {
				auth = authInterface.checkToken(authToken);
			} catch (IOException e) {
				return false;
			} catch (SAXException e) {
				return false;
			} catch (FlickrException e) {
				return false;
			}
			RequestContext.getRequestContext().setAuth(auth);

			return true;
		}
	}

	// The stored token didn't work (or there was no stored token)
	// the user has to give his permission to us to access his data
	// we don't need this here
	private boolean doAuth() throws IOException, SAXException {
		String frob = null;
		try {
			frob = authInterface.getFrob();
		} catch (FlickrException e) {
			e.printStackTrace();
		}
		System.out.println("frob: " + frob);

		URL url = authInterface.buildAuthenticationUrl(Permission.READ, frob);

		Thread launcherThread = null;

		try {
			BrowserLauncher launcher = new BrowserLauncher(null);
			BrowserLauncherRunner runner = new BrowserLauncherRunner(launcher,
					url.toExternalForm(), null);
			launcherThread = new Thread(runner);
			launcherThread.start();
		} catch (Exception ex) {
		}

		System.out
				.println((new StringBuilder())
						.append(
								"A web browser should popup. If not, copy paste the following in a web browser :")
						.append(url.toExternalForm())
						.append(
								" and authenticate the JFlickrUploader application and press ENTER.")
						.toString());
		System.in.read();

		try {
			Auth auth = authInterface.getToken(frob);
			authToken = auth.getToken();
			RequestContext.getRequestContext().setAuth(auth);
			return true;
		} catch (FlickrException e) {
			System.out.println("Authentication failed");
			return false;
		}
	}

	// load the Flickr properties
	private void loadProperties() throws IOException {
		FileInputStream propInFile = new FileInputStream(filename);
		properties.load(propInFile);

		apiKey = properties.getProperty("apiKey");
		sharedSecret = properties.getProperty("secret");
		authToken = properties.getProperty("token");
	}

	// save the Flickr properties
	private void saveProperties() throws IOException {
		properties.setProperty("apiKey", apiKey);
		properties.setProperty("secret", sharedSecret);
		properties.setProperty("token", authToken);

		FileOutputStream propOutFile = new FileOutputStream(filename);
		properties.store(propOutFile, null);
	}

	// This method returns a buffered image with the contents of an image
	public static BufferedImage toBufferedImage(Image image) {
		if (image instanceof BufferedImage) {
			return (BufferedImage) image;
		}

		// This code ensures that all the pixels in the image are loaded
		image = new ImageIcon(image).getImage();

		// Determine if the image has transparent pixels; for this method's
		// implementation, see e661 Determining If an Image Has Transparent
		// Pixels
		// boolean hasAlpha = hasAlpha(image);
		boolean hasAlpha = false;

		// Create a buffered image with a format that's compatible with the
		// screen
		BufferedImage bimage = null;
		GraphicsEnvironment ge = GraphicsEnvironment
				.getLocalGraphicsEnvironment();
		try {
			// Determine the type of transparency of the new buffered image
			int transparency = Transparency.OPAQUE;
			if (hasAlpha) {
				transparency = Transparency.BITMASK;
			}

			// Create the buffered image
			GraphicsDevice gs = ge.getDefaultScreenDevice();
			GraphicsConfiguration gc = gs.getDefaultConfiguration();
			bimage = gc.createCompatibleImage(image.getWidth(null), image
					.getHeight(null), transparency);
		} catch (HeadlessException e) {
			// The system does not have a screen
		}

		if (bimage == null) {
			// Create a buffered image using the default color model
			int type = BufferedImage.TYPE_INT_RGB;
			if (hasAlpha) {
				type = BufferedImage.TYPE_INT_ARGB;
			}
			bimage = new BufferedImage(image.getWidth(null), image
					.getHeight(null), type);
		}

		// Copy image to buffered image
		Graphics g = bimage.createGraphics();

		// Paint the image onto the buffered image
		g.drawImage(image, 0, 0, null);
		g.dispose();

		return bimage;
	}

	public static void main(String[] args) {
		try {
			FlickrTest app = new FlickrTest();
		} catch (Exception e) {
			e.printStackTrace();
		}
		System.exit(0);
	}
}
