import java.util.*;

/**
 * An implemented user, extends the abstract Graphnode.
 */
public class FlickrUser extends GraphNode<FlickrUser> {
	public int nrPhotoSets;

	public int nrTags;

	// private ArrayList<FlickrUser> contacts;
	private String userName;

	public static final int TAGS = 0;

	public static final int PHOTOSETS = 1;

	public static final int CONTACTS = 2;

	public static final int CONSTANT = -1;

	private int myNodeSize = PHOTOSETS;

	public FlickrUser(String userName) {
		this.userName = userName;
		contacts = new ArrayList<FlickrUser>();
		nrPhotoSets = 0;
		nrTags = 0;
	}

	/**
	 * adds an contact to the user
	 * @param contact is added if not already in list
	 */
	public void addContact(FlickrUser contact) {
		if (contacts.contains(contact))
			return;
		contacts.add(contact);
		contact.addContact(this);
	}

	public ArrayList<FlickrUser> getContacts() {
		return contacts;
	}

	public String getUserName() {
		return userName;
	}

	public void setNodeSize(int type) {
		myNodeSize = type;
	}

	/**
	 * returns the usersize of this node, 
	 * depending on the different settings. 
	 */
	public int getUserSize() {
		switch (myNodeSize) {
		case TAGS:
			return clampSize(nrTags, 10, 200);
		case PHOTOSETS:
			return clampSize(nrPhotoSets, 0, 10);
		case CONTACTS:
			return clampSize(getBelowUsers(), 0, 10);
		default:
			return 30;
		}

	}
	/**
	 * clamps a value to a given threshold
	 * @param size to be clamped
	 * @param lowerThreshold 
	 * @param upperThreshold
	 * @return
	 */
	private int clampSize(int size, int lowerThreshold, int upperThreshold) {
		int minSize = 20;
		int maxSize = 50;
		if (size <= lowerThreshold)
			return minSize;
		else if (size >= upperThreshold)
			return maxSize;
		else {
			double scale = (double) (size - lowerThreshold)
					/ (double) (upperThreshold - lowerThreshold);
			return (int) (scale * (maxSize - minSize) + minSize);
		}
	}

	public String toString() {
		return userName + " " + nrPhotoSets + " " + nrTags + contacts.size();
	}

	
	private int getBelowUsers() {
		int count = 0;
		int myDepth = bfs.getSearchResult().get(this);
		for (FlickrUser contact : contacts) {
			if (bfs.getSearchResult().get(contact) > myDepth)
				count += 1 + contact.getBelowUsers();
		}
		return count;
	}
}
