import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

/**
 * loads all users
 *
 */
public class FlickrUserData {
	HashMap<String, FlickrUser> userList;

	@SuppressWarnings("unchecked")
	public FlickrUserData() {
		userList = new HashMap();
	}

	public boolean containsUser(String userName) {
		return userList.containsKey(userName);
	}

	public FlickrUser getUser(String userName) {
		return userList.get(userName);
	}

   public void addUser(FlickrUser user){
      addUser(user.getUserName(),user);
   }
	public void addUser(String userName, FlickrUser user) {
		userList.put(userName, user);
	}

	public int size() {
		return userList.size();
	}

	public void printDebug() {
		for (String userName : userList.keySet()) {
			FlickrUser u = userList.get(userName);

			System.out.println(userName + "   tags: " + u.nrTags + " sets: "
					+ u.nrPhotoSets + " contacts: " + u.getContacts().size());

			for (FlickrUser contact : u.getContacts()) {
				System.out.println("* " + contact.getUserName());
			}
		}
	}

	// Saves the data to XML files
	//
	// data/users.xml contains a list of all users
	// example:
	// <flickr>
	// <name>Bombarde32</name>
	// <name>TheWulf</name>
	// <name>mbuchetics</name>
	// <name>apieceofdenise</name>
	// <name>Chris.si</name>
	// </flickr>
	//
	// data/*username*/data.xml contains information for the specific user
	// example: 
	// <data>
	// <nrTags>184</nrTags>
	// <nrPhotoSets>9</nrPhotoSets>
	// <contact>TheWulf</contact>
	// <contact>apieceofdenise</contact>
	// </data>
	public void save() throws IOException {
		Document docUsers = new Document();
		Element docUsersRoot = new Element("flickr");
		docUsers.addContent(docUsersRoot);

		for (FlickrUser user : userList.values()) {
			// for every user create a name element
			Element name = new Element("name");
			docUsersRoot.addContent(name);
			name.addContent(user.getUserName());

			// create a new xml document for every user (data.xml)
			Document docUserData = new Document();
			Element docUserDataRoot = new Element("data");
			docUserData.addContent(docUserDataRoot);

			// add the number of tags
			Element nrTags = new Element("nrTags");
			nrTags.addContent(Integer.toString(user.nrTags));
			docUserDataRoot.addContent(nrTags);

			// add the number of photosets
			Element nrPhotoSets = new Element("nrPhotoSets");
			nrPhotoSets.addContent(Integer.toString(user.nrPhotoSets));
			docUserDataRoot.addContent(nrPhotoSets);

			// iterate over all the contacts and add them as a contact element
			for (FlickrUser contact : user.getContacts()) {
				Element cont = new Element("contact");
				cont.addContent(contact.getUserName());
				docUserDataRoot.addContent(cont);
			}

			// save the data.xml file
			FileOutputStream userData = new FileOutputStream("data/" + user.getUserName()
					+ "/data.xml");
			XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
			out.output(docUserData, userData);
		}

		// save the users.xml file
		FileOutputStream usersFile = new FileOutputStream("data/users.xml");
		XMLOutputter out = new XMLOutputter(Format.getPrettyFormat());
		out.output(docUsers, usersFile);
	}

	// Loads the data back from the XML files
	// returns false if something goes wrong, true if everything works
	// see save() for more information on the XML file structure
	public boolean load() {
		SAXBuilder builder = new SAXBuilder();
		
		// load users.xml
		Document docUsers;
		try {
			docUsers = builder.build("data/users.xml");
		} catch (Exception e) {
			e.printStackTrace();
			return false;
		}

		Element docUsersRoot = docUsers.getRootElement();
		Iterator users = docUsersRoot.getChildren("name").iterator();

		// iterate over all the names
		while (users.hasNext()) {
         loadUser((Element) users.next());
		}
		return true;
	}
   
	/**
	 * worker method for loading users
	 * @param userName
	 */
   private void loadUser(Element userName){
     if(containsUser(userName.getValue()))
        return;
      FlickrUser newUser = new FlickrUser(userName.getValue());

      // store the name and the user object in the hashmap
      addUser(newUser);

      // load the data.xml for the specific user
      Document docUserData;
      try {
         docUserData = new SAXBuilder().build("data/" + userName.getValue()
               + "/data.xml");
      } catch (Exception e) {
         e.printStackTrace();
         return ;
      }

      // load all the user information and store it in its user object
      Element docUserDataRoot = docUserData.getRootElement();
      newUser.nrTags = Integer.parseInt(docUserDataRoot
            .getChild("nrTags").getValue());
      newUser.nrPhotoSets = Integer.parseInt(docUserDataRoot.getChild(
            "nrPhotoSets").getValue());

      Iterator contacts = docUserDataRoot.getChildren("contact")
            .iterator();

      // iterate over all contacts and store them in the arraylist
      while (contacts.hasNext()) {
         Element contactName = (Element) contacts.next();
         //if(!containsUser(contactName.getValue()))
         loadUser(contactName);
         newUser.addContact(getUser(contactName.getValue()));
      }
   }
}
