import java.util.HashMap;

import javax.swing.JFrame;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.event.PBasicInputEventHandler;
import edu.umd.cs.piccolo.event.PInputEvent;
import edu.umd.cs.piccolo.nodes.PText;

/**
 * starting class
 * @author fm
 *
 */
public class RadialGraph extends MyPFrame {

	private HashMap<String, PNode> createdNodes = null;

	private PLayer nodeLayer;

	private PLayer edgeLayer;

	private PLayer circleLayer;

	private PText infoText;

	int lastX = 0;

	int lastY = 0;

	boolean rebuildGraph = false;

	private RadialGraphBuilder graphBuilder = null;

	public RadialGraph() {
		this(null);
	}

	public RadialGraph(PCanvas aCanvas) {
		super("FlickrPiccoloTest", false, aCanvas);
		createdNodes = new HashMap<String, PNode>();
	}

	public void initialize() {

		// this.doLayout();
		// Initialize, and create a layer for the edges (always underneath the
		// nodes)
		nodeLayer = getCanvas().getLayer();
		edgeLayer = new PLayer();
		circleLayer = new PLayer();
		infoText = new PText();

		getCanvas().getCamera().addLayer(0, circleLayer);
		getCanvas().getCamera().addLayer(1, edgeLayer);

		// rotate layers so the focus node looks right
		nodeLayer.rotateAboutPoint(Math.toRadians(90),
				getCanvas().getWidth() / 2.0f, getCanvas().getHeight() / 2.0f);
		edgeLayer.rotateAboutPoint(Math.toRadians(90),
				getCanvas().getWidth() / 2.0f, getCanvas().getHeight() / 2.0f);

		infoText.setText("");
		infoText.setScale(3.0f);
		infoText.setX(5.0f);
		infoText.setY(240.0f);

		// attach it to camera, make it "sticky"
		getCanvas().getCamera().addChild(infoText);


		
		nodeLayer.addInputEventListener(new InputHandler());
		FlickrUserData userData = new FlickrUserData();

		// load all the data from XML
		userData.load();

		graphBuilder = new RadialGraphBuilder(getCanvas(), nodeLayer,
				edgeLayer, circleLayer);
		
		JFrame frame = new JFrame();
		frame.setContentPane(new ControlPanel(graphBuilder));
		frame.pack();
		frame.setVisible(true);
		graphBuilder.buildGraph(userData.getUser("droobles"));
	}

	public static void main(String[] args) {
		new RadialGraph();
	}

	class InputHandler extends PBasicInputEventHandler {
		public void mouseEntered(PInputEvent event) {
			if (graphBuilder == null || graphBuilder.animationRunning == true)
				return;

			PNode node = event.getPickedNode();
			FlickrUser user = (FlickrUser) node
					.getAttribute(RadialGraphBuilder.USERDATA);

			if (!user.isHighlighted()) {
				node.scaleAboutPoint(2.0f, node.getBounds().getCenterX(), node
						.getBounds().getCenterY());
				user.setHighlighted(true);
			}

			graphBuilder.unmarkEdges();
			graphBuilder.markEdges(node);

			infoText.setText(user.getUserName());// + " " + user.nrTags);
			infoText.repaint();
		}

		public void mouseExited(PInputEvent event) {
			if (graphBuilder == null || graphBuilder.animationRunning)
				return;

			PNode node = event.getPickedNode();
			FlickrUser user = (FlickrUser) node
					.getAttribute(RadialGraphBuilder.USERDATA);

			if (user.isHighlighted()) {
				node.scaleAboutPoint(0.5f, node.getBounds().getCenterX(), node
						.getBounds().getCenterY());
				user.setHighlighted(false);
			}

			graphBuilder.unmarkEdges();

			infoText.setText("");
			infoText.repaint();
		}

		public void mousePressed(PInputEvent event) {
			if (graphBuilder == null || graphBuilder.animationRunning == true)
				return;

			PNode node = event.getPickedNode();
			FlickrUser user = (FlickrUser) node
					.getAttribute(RadialGraphBuilder.USERDATA);

			if (user == graphBuilder.oldFocus)
				return;

			if (user.isHighlighted()) {
				node.scaleAboutPoint(0.5f, node.getBounds().getCenterX(), node
						.getBounds().getCenterY());
				user.setHighlighted(false);
			}

			// graphBuilder.removeGraph();
			graphBuilder.buildGraph(user);
		}
	}
}
