import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Queue;
import java.util.Timer;

import com.sun.jmx.snmp.tasks.Task;

import edu.umd.cs.piccolo.PCanvas;
import edu.umd.cs.piccolo.PLayer;
import edu.umd.cs.piccolo.PNode;
import edu.umd.cs.piccolo.nodes.PImage;
import edu.umd.cs.piccolo.nodes.PPath;

/**
 * creates a radial graph *
 */
public class RadialGraphBuilder {
	private HashMap<GraphNode, PNode> createdNodes = null;

	private ArrayList<PPath> markedEdges = null;

	private PLayer nodeLayer;

	private PLayer edgeLayer;

	private PLayer circleLayer;

	private PCanvas canvas;

	private int radius = 100;

	public GraphNode oldFocus = null;

	private double offset = 0;

	public boolean animationRunning = false;

	public static final String USERDATA = "userdata";

	public static final String EDGES = "edges";

	public static final String NODES = "nodes";

	private float fisheye = 1.0f;

	private int numCircles = 15;

	private int circleDepths[];

	private boolean chromaDepth = false;

	public RadialGraphBuilder(PCanvas canvas, PLayer nodeLayer,
			PLayer edgeLayer, PLayer circleLayer) {
		this.nodeLayer = nodeLayer;
		this.edgeLayer = edgeLayer;
		this.circleLayer = circleLayer;
		this.canvas = canvas;
		createdNodes = new HashMap<GraphNode, PNode>();
		markedEdges = new ArrayList<PPath>();
	}

	// public void removeGraph()
	// {
	// nodeLayer.removeAllChildren();
	// edgeLayer.removeAllChildren();
	// circleLayer.removeAllChildren();
	// createdNodes.clear();
	// }

	/** 
	 * rebuild our graph
	 * needed when fisheye or radius changed 
	 */
	public void rebuildGraph() {
		buildGraph( oldFocus);
	}

	/**
	 * create a graph from a startingnode
	 * @param startNode
	 */
	public void buildGraph(GraphNode startNode) {
		// GraphNode.clearVisitedNodes();
		if (oldFocus == null)
			createCircles();
		double targetAngle = 0;
		startNode.startBFS();
		startNode.resetCalculated();
		startNode.getAngularWidth(null);
		// createSubgraph(startUser, 0, 360);

		if (oldFocus != null) {
			double startAngle = startNode.getAngle();
			targetAngle = startAngle - 180;
		}

		startNode.calculatePosition(0, 360);

		if (oldFocus != null) {
			offset = targetAngle - oldFocus.getAngle();
			Timer timer = new Timer();
			AnimationTask myTask = new AnimationTask(createdNodes, this);
			timer.schedule(myTask, 0, 100);
		} else {
			// createCircles();
			createNodes();
		}

		oldFocus = startNode;
	}

	/**
	 * draw circles 
	 * @param fisheye
	 */
	public void createCircles(float fisheye) {
		this.fisheye = fisheye;
		createCircles();
	}

	public void createCircles(int depthIncrease) {
		this.radius = depthIncrease;
		createCircles();
	}

	public void createCircles(float fisheye, int depthIncrease) {
		this.fisheye = fisheye;
		this.radius = depthIncrease;
		createCircles();
	}

	private void createCircles() {
		circleLayer.removeAllChildren();
		circleDepths = new int[numCircles];
		float circleSize = 0;
		circleDepths[0] = 0;
		for (int i = 1; i < numCircles - 1; i++) {
			circleSize = circleSize + (float) (radius * Math.pow(fisheye, i));
			circleDepths[i] = (int) circleSize;
			PPath node = PPath.createEllipse(
					(float) (canvas.getWidth() / 2.0f - circleDepths[i]),
					(float) (canvas.getHeight() / 2.0f - circleDepths[i]),
					circleDepths[i] * 2, circleDepths[i] * 2);

			node.setStroke(new BasicStroke(0.3f));
			node.setPaint(null);
			circleLayer.addChild(node);
		}
		GraphNode.circleDepths = circleDepths;
		circleLayer.repaint();
	}

	private void createNodes() {
		Queue<GraphNode> q = GraphNode.bfs.getWorkList();
		for (GraphNode element : q) {
			element.animateTo(element.getGoalDepth(), element.getGoalAngle()
					+ offset);
			element.finishAnimation();

			createUserNode(element);
		}

		// TODO: each edge is added twice!!!
		// Doppelte entfernen!!!!!!!!
		for (GraphNode element : q) {
			for (Iterator iter = element.getNeighbours(); iter.hasNext();)
				addEdge(element, (GraphNode) iter.next());
		}
	}

	/**
	 * reposition all nodes 
	 * (for example after fisheye or radius change)
	 *
	 */
	public void updateNodes() {
		for (GraphNode gNode : createdNodes.keySet()) {
			PNode pNode = createdNodes.get(gNode);
			// Point pos = calcXY(calcDistance(gNode), gNode.getAngle());
			Point pos = calcXY((gNode.getDepth()), gNode.getAngle());

			// rotate back to 0, reset rotation
			pNode.rotateAboutPoint(-pNode.getRotation(), pNode.getBounds()
					.getCenterX(), pNode.getBounds().getCenterY());

			// pNode.setX(pos.getX() - gNode.getUserSize() / 2.0f);
			// pNode.setY(pos.getY() - gNode.getUserSize() / 2.0f);

			pNode.setBounds(pos.getX() - gNode.getUserSize() / 2.0f, pos.getY()
					- gNode.getUserSize() / 2.0f, gNode.getUserSize(), gNode
					.getUserSize());

			pNode.rotateAboutPoint(Math.toRadians(90 + gNode.getAngle()), pos
					.getX(), pos.getY());

			if (chromaDepth == true) {
				pNode.setTransparency(Math.max(0.2f, 1.0f
						- (float) gNode.getGraphDepth() / (numCircles / 1.5f)));
			} else {
				pNode.setTransparency(1.0f);
			}

		}

		for (Iterator iter = edgeLayer.getChildrenIterator(); iter.hasNext();) {
			PPath anEdge = (PPath) iter.next();
			ArrayList<PNode> nodes = getAttribute(anEdge, NODES);
			PNode n1 = nodes.get(0);
			PNode n2 = nodes.get(1);
			anEdge.reset();
			anEdge.moveTo((float) n1.getBounds().getCenterX(), (float) n1
					.getBounds().getCenterY());
			anEdge.lineTo((float) n2.getBounds().getCenterX(), (float) n2
					.getBounds().getCenterY());
		}

		nodeLayer.repaint();
		edgeLayer.repaint();
	}

	/**
	 * caluclate the coordinates for given depth and angle
	 * @param depth
	 * @param phi
	 * @return
	 */
	private Point calcXY(double depth, double phi) {
		// if(useFisheye)

		float circleSize = 0;
		// for(int i = 0; i<depth; i++)
		circleSize = (float) depth;// circleSize + (float)DEPTHINCREASE )*
		// fisheye;
		// depth = (int) (Math.pow(fisheye,depth) *DEPTHINCREASE);
		int xOffset = canvas.getWidth() / 2;
		int yOffset = canvas.getHeight() / 2;
		return new Point((int) (circleSize * Math.cos(Math.toRadians(phi)))
				+ xOffset, // X coordinate on the unit sphere, offset to the
				// middle
				(int) (circleSize * Math.sin(Math.toRadians(phi))) + yOffset); // and
		// the
		// Y
	}

	private double calcDistance(GraphNode node) {
		return node.getDepth() * radius;
	}

	/**
	 * get a graphical instance of the usernode
	 * @param user
	 * @return a piccolo object
	 */
	private PNode createUserNode(GraphNode user) {
		return createFlickrUserNode(user);
	}

	/**
	 * created a node for a flickruser
	 * @param user
	 * @return
	 */
	private PNode createFlickrUserNode(GraphNode user) {
		// Point pos = calcXY(calcDistance(user), user.getAngle());
		Point pos = calcXY((user.getDepth()), user.getAngle());

		PImage node = new PImage("data/" + ((FlickrUser) user).getUserName()
				+ "/icon.jpg");
		node.setBounds(pos.getX() - user.getUserSize() / 2.0f, pos.getY()
				- user.getUserSize() / 2.0f, user.getUserSize(), user
				.getUserSize());

		node.rotateAboutPoint(Math.toRadians(90 + user.getAngle()), pos.getX(),
				pos.getY());

		node.addAttribute(USERDATA, user);
		node.addAttribute(EDGES, new ArrayList<PNode>());
		createdNodes.put(user, node);
		nodeLayer.addChild(node);
		return node;
	}

	/**
	 * add edges between two nodes
	 * @param n1
	 * @param n2
	 */
	private void addEdge(GraphNode n1, GraphNode n2) {
		addEdge(createdNodes.get(n1), createdNodes.get(n2));
	}

	private void addEdge(PNode n1, PNode n2) {
		if (n1 == null || n2 == null)
			return;

		PPath edge = new PPath();
		edge.moveTo((float) n1.getBounds().getCenterX(), (float) n1.getBounds()
				.getCenterY());
		edge.lineTo((float) n2.getBounds().getCenterX(), (float) n2.getBounds()
				.getCenterY());

		edge.setStroke(new BasicStroke(0.4f));

		ArrayList<PNode> myNodes = new ArrayList<PNode>();
		myNodes.add(n1);
		myNodes.add(n2);
		edge.addAttribute(NODES, myNodes);

		getAttribute(n1, EDGES).add(edge);
		getAttribute(n2, EDGES).add(edge);
		edgeLayer.addChild(edge);
	}

	/**
	 * shortcut to get attributes...
	 */
	private ArrayList<PNode> getAttribute(PNode node, Object key) {
		return (ArrayList<PNode>) node.getAttribute(key);
	}

	/**
	 * mark and highlight edges from a node
	 * @param node all edges of this node are beeing highlighted
	 */
	public void markEdges(PNode node) {
		ArrayList<PNode> edges = getAttribute(node, RadialGraphBuilder.EDGES);

		for (Iterator iter = edges.iterator(); iter.hasNext();) {
			PPath edge = (PPath) iter.next();

			edge.setStroke(new BasicStroke(2.5f));
			edge.setStrokePaint(Color.red);

			markedEdges.add(edge);
		}
	}

	/**
	 * unmark ALL edges
	 *
	 */
	public void unmarkEdges() {
		for (PPath edge : markedEdges) {
			edge.setStroke(new BasicStroke(0.4f));
			edge.setStrokePaint(Color.black);
		}

		markedEdges.clear();
	}

	/**
	 * set the nodesize of all nodes to a given value
	 */
	public void setNodeSizeMethod(int value) {
		for (GraphNode gNode : createdNodes.keySet()) {
			FlickrUser user = (FlickrUser) gNode;
			user.setNodeSize(value);
		}

		// updateNodes();
		rebuildGraph();
	}
	
	public void setChromaDepth(boolean value) {
		if(chromaDepth != value) {
			chromaDepth = value;
			updateNodes();
		}
	}
}
